
const shareData={
    jwt:"",
    auth_code:""
}
function getClusters(indexRef, zoom, {northEast, southWest}) {
    zoom = Math.round(zoom)
    const {lat: northLat, lng: eastLng} = northEast
    const {lat: southLat, lng: westLng} = southWest
    const clusters = indexRef.getClusters([westLng, southLat, eastLng, northLat], zoom)
    let result = []
    for (let i = 0; i < clusters.length; i++) {
        const {id, properties, geometry} = clusters[i];
        if (properties.cluster) {
            const items = indexRef.getLeaves(id, 1, 0)
            const first = items[0]
            result.push({
                id: first.properties.id,
                cluster_id: id,
                latitude: geometry.coordinates[1],
                longitude: geometry.coordinates[0],
                md5: "",
                count: properties.point_count,
            })
        } else {
            result.push({
                id: properties.id,
                latitude: geometry.coordinates[1],
                longitude: geometry.coordinates[0],
                count: 1,
                md5: ""
            })
        }
    }
    return result
}

/**
 * 根据本地计算的汇聚点id，查询 MD5、autoNavi信息
 * @param list
 */
function getIdsMD5(list) {
    return new Promise((resolve, reject) => {
        const ids = list.map((a) => a.id)
        gpsGroupAreaFilesMD5(ids).then((res) => {
            let idMaps = {}
            for (let i = 0; i < res.length; i++) {
                const {id, MD5, gpsInfo} = res[i];
                idMaps[id] = {MD5, autoNavi: gpsInfo && gpsInfo.autoNavi}
            }
            for (let i = 0; i < list.length; i++) {
                const {id} = list[i];
                let info = idMaps[id]
                if (info) {
                    const {MD5} = info
                    list[i].md5 = MD5
                }
            }
            resolve(list)
        }).catch(reject)
    })
}

function gpsGroupAllFiles() {
    return mtFetch('/gateway/allFilesForMap', "", "GET")
}
function gpsMapboxToken() {
    return mtFetch('/gateway/mapboxToken', "", "GET")
}
function gpsMaptilerToken() {
    return mtFetch('/gateway/maptilerToken', "", "GET")
}
function gpsGroupAllFilesDirect() {
    return mtFetch('/gateway/allFilesForMapDirect', "", "GET")
}

function gpsGroupAreaFilesMD5(ids) {
    return mtFetch('/gateway/areaFilesMD5', {ids}, "POST")
}

function getAccessToken() {
    const ACCESS_TOKEN_SAVE_KEY = '__access_token';
    return new Promise((resolve, reject) => {
        if(shareData.jwt){
            resolve(shareData.jwt)
            return
        }
        let l = localStorage.getItem(ACCESS_TOKEN_SAVE_KEY);
        let s = sessionStorage.getItem(ACCESS_TOKEN_SAVE_KEY);
        const data = JSON.parse(s || l);
        if (data && data.access_token) {
            return resolve(data.access_token);
        }
        resolve("")
    });
}

function mtFetch(url, param = {}, method = 'GET') {
    const TIMEOUT = 30 * 1000  //超时时间
    if (url === undefined) {
        return Promise.reject("请求地址为空")
    }

    return new Promise(function (resolve, reject) {
        setTimeout(() => {
            reject('网络请求超时，请检查网络设置')
        }, TIMEOUT)

        getAccessToken().then((accessToken) => {
            _fetch(accessToken)
        }).catch((e) => {
            console.log("catch error:", e)
        })

        function _fetch(accessToken) {
            let fetch_url = url
            let fetch_param = {
                method,
                credentials: "same-origin",
                headers: {
                    "Content-Type": "application/json",
                },
                body: JSON.stringify(param)
            }
            if (accessToken) {
                fetch_param.headers.jwt = accessToken
            }
            if (method === "GET") {
                let requestParam = param
                if (fetch_url.search(/\?/) === -1) {
                    fetch_url += '?' + requestParam
                } else {
                    fetch_url += '&' + requestParam
                }
                delete fetch_param.body
            }
            let responseStatus = 200
            fetch(fetch_url, fetch_param)
                .then((response) => {
                    responseStatus = response.status
                    if (responseStatus >= 500) {
                        return response.text()
                    }
                    return response.json()
                })
                .then((response) => {
                    if (responseStatus >= 500) {
                        try {
                            response = JSON.parse(response)
                        } catch (e) {

                        }
                    }
                    if (response) {
                        if (response.code) {
                            return reject(response)
                        }
                        if (response.message && response.statusCode) {
                            return reject(response)
                        }
                    }
                    if (responseStatus >= 500) {
                        return reject({statusCode: responseStatus, msg: response})
                    }
                    if (responseStatus > 299) {
                        return reject(response)
                    }
                    resolve(response)
                })
                .catch((err) => {
                    reject({statusCode: responseStatus, msg: err.toString()})
                })
        }
    });
}

function getRequestErrorMsg(err){
    console.log("err", err)
    if(!err){
        return "unknown error"
    }
    if(err.msg){
        return err.msg
    }
    if(err.message){
        return err.message
    }
    if(err.code){
        return err.code
    }
    return err.toString()
}

const centerSaveKey = "MAP_MOVEED_CENTER"

function saveCenter(map) {
    const zoom = map.getZoom();
    const center = map.getCenter();
    sessionStorage.setItem(centerSaveKey, `${center.lng},${center.lat},${zoom}`)
}

function getCenter(defaultCenter, zoom) {
    const str = sessionStorage.getItem(centerSaveKey)
    if (str && str.length > 0) {
        let part = str.split(",")
        if (part.length === 3) {
            return {center: [part[0], part[1]], zoom: part[2]}
        }
    }
    return {center: defaultCenter, zoom}
}

function getUrlParams() {
    const queryString = window.location.search;
    const params = new URLSearchParams(queryString);
    // if (params && params.get) {
    //     const jwt = params.get("jwt")
    //     const auth_code = params.get("auth_code")
    //     if (jwt && auth_code) {
    //         shareData.auth_code = auth_code
    //         shareData.jwt = jwt
    //     }
    // }
    return params;
}

function getUrlParamNumber(urlParams, key) {
    let str = urlParams.get(key)
    if (str) {
        return Number(str)
    }
    return str
}

function loadImg(imgElement, src) {
    let auth_code = shareData.auth_code
    const options = {
        headers: {
            'x-mt-auth-code': auth_code
        }
    };

    fetch(src, options)
        .then(res => res.blob())
        .then(blob => {
            imgElement.src = URL.createObjectURL(blob);
        });
}

function filterLayers(worldview) {
    // The "admin-0-boundary-disputed" layer shows boundaries
    // at this level that are known to be disputed.
    map.setFilter('admin-0-boundary-disputed', [
        'all',
        ['==', ['get', 'disputed'], 'true'],
        ['==', ['get', 'admin_level'], 0],
        ['==', ['get', 'maritime'], 'false'],
        ['match', ['get', 'worldview'], ['all', worldview], true, false]
    ]);
    // The "admin-0-boundary" layer shows all boundaries at
    // this level that are not disputed.
    map.setFilter('admin-0-boundary', [
        'all',
        ['==', ['get', 'admin_level'], 0],
        ['==', ['get', 'disputed'], 'false'],
        ['==', ['get', 'maritime'], 'false'],
        ['match', ['get', 'worldview'], ['all', worldview], true, false]
    ]);
    // The "admin-0-boundary-bg" layer helps features in both
    // "admin-0-boundary" and "admin-0-boundary-disputed" stand
    // out visually.
    map.setFilter('admin-0-boundary-bg', [
        'all',
        ['==', ['get', 'admin_level'], 0],
        ['==', ['get', 'maritime'], 'false'],
        ['match', ['get', 'worldview'], ['all', worldview], true, false]
    ]);
}

function filterLayersMaptiler(map) {
    map.removeLayer("Disputed border");
    map.removeLayer("Other border");
    map.addLayer({
        "id": "CN border (disputed)",
        "type": "line",
        "source": "maptiler_planet",
        "source-layer": "boundary",
        "layout": {"visibility": "visible"},
        "paint": {"line-color": "hsl(0, 0%, 54%)", "line-width": {"stops": [[1, 0.5], [5, 1.5], [10, 2], [24, 12]]}},
        "filter": ["all", ["<=", ["get", "admin_level"], 2], ["==", ["geometry-type"], "LineString"], ["==", ["get", "disputed"], 1], ["==", ["get", "claimed_by"], "CN"]]
    });
    map.addLayer({
        "id": "Disputed border",
        "type": "line",
        "source": "maptiler_planet",
        "source-layer": "boundary",
        "minzoom": 0,
        "layout": {"line-cap": "round", "line-join": "round", "visibility": "visible"},
        "paint": {
            "line-color": "hsl(0, 0%, 63%)",
            "line-dasharray": [2, 2],
            "line-width": {"stops": [[1, 0.5], [5, 1.5], [10, 2], [24, 12]]}
        },
        "filter": ["all", ["==", "$type", "LineString"], ["all", ["==", "admin_level", 2], ["==", "disputed", 1], ["==", "maritime", 0], ["any", ["in", "claimed_by", "BF", "BJ", "BT", "CN", "EG", "EH", "ES", "ET", "FR", "IL", "IT", "KE", "KG", "NP", "PK", "PS", "RU", "SD", "SS", "SY", "TR", "UA", "UN", "XA", "XK", "XS"], ["!has", "claimed_by"]]]]
    })
}
function getMapboxConfig(lang){
    return {
        language:"en",
    }
}
function getMaptilerConfig(lang){
    return {
    }
}
